"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const postcss_1 = __importDefault(require("postcss"));
const parser_1 = __importDefault(require("./parser"));
const postcss_nested_1 = __importDefault(require("postcss-nested"));
const CONFIG_PATH = process.env.JEST_CSS_MODULES_TRANSFORM_CONFIG || 'jest-css-modules-transform-config.js';
const postcssNested = postcss_1.default([postcss_nested_1.default]);
// eslint-disable-next-line @typescript-eslint/no-explicit-any
let postcssConfig;
let postcssPluginWithConfig = null;
let stylus;
let sass;
let less;
const moduleTemplate = `
    "use strict";

    const data = %s;

    if (typeof module === 'object' && module) {
        Object.defineProperty(data, "__esModule", {
            value: true
        });
        module.exports = new Proxy(data, {
            get(target, attr) {
                if (attr === 'default') {
                    return target;
                };
                return target[attr];
            },

            getPrototypeOf() {
                return Object;
            }
        });
    } else {
        Object.defineProperty(exports, "__esModule", {
            value: true
        });
        exports.default = data;
    }
`;
let preProcessorsConfig;
const getPreProcessorsConfig = (function wrap() {
    const preProcessorsConfigDefalut = {
        sassConfig: {},
        lessConfig: {},
        stylusConfig: {},
        cssLoaderConfig: {},
    };
    return (rootDir) => {
        if (preProcessorsConfig) {
            return preProcessorsConfig;
        }
        try {
            return require(path_1.resolve(rootDir, CONFIG_PATH));
        }
        catch (e) {
            return preProcessorsConfigDefalut;
        }
    };
}());
let globalSassData;
const getGlobalSassData = (rootDir) => {
    try {
        return `${require(path_1.resolve(rootDir, '.sassrc.js')).data}\n` || '';
    }
    catch (e) {
        return '';
    }
};
const requirePostcssConfig = (rootDir) => {
    try {
        return require(path_1.resolve(rootDir, 'postcss.config.js')) || null;
    }
    catch (e) {
        return null;
    }
};
const getFileExtension = (path) => {
    const filename = path.slice(path.lastIndexOf(path_1.sep) + 1);
    return filename.slice(filename.lastIndexOf('.') + 1);
};
const getSassContent = (src, path, extention, rootDir) => {
    sass = sass || require('node-sass');
    globalSassData = globalSassData === undefined ? getGlobalSassData(rootDir) : globalSassData;
    const sassConfig = Object.assign(preProcessorsConfig.sassConfig || {}, {
        data: globalSassData + src,
        file: path,
        indentedSyntax: extention === 'sass',
    });
    return String(sass.renderSync(sassConfig).css);
};
let parser;
const moduleTransform = {
    process(src, path, config) {
        preProcessorsConfig = preProcessorsConfig || getPreProcessorsConfig(config.rootDir);
        parser = parser || new parser_1.default(preProcessorsConfig.cssLoaderConfig);
        const extention = getFileExtension(path);
        let textCSS = src;
        let lessConfig;
        let stylusConfig;
        switch (extention) {
            case 'styl':
                stylus = stylus || require('stylus');
                stylusConfig = Object.assign(preProcessorsConfig.stylusConfig || {}, { filename: path });
                stylus.render(src, stylusConfig, (err, css) => {
                    if (err) {
                        throw err;
                    }
                    textCSS = css;
                });
                break;
            case 'sass':
            case 'scss':
                textCSS = getSassContent(src, path, extention, config.rootDir);
                break;
            case 'less':
                less = less || require('less');
                lessConfig = Object.assign(preProcessorsConfig.lessConfig || {}, { filename: path });
                less.render(src, lessConfig, (err, css) => {
                    if (err) {
                        throw err;
                    }
                    textCSS = css.css;
                });
                break;
            case 'css':
            case 'pcss':
            case 'postcss':
                postcssConfig = postcssConfig || preProcessorsConfig.postcssConfig || requirePostcssConfig(config.rootDir);
                if (postcssConfig) {
                    postcssPluginWithConfig = postcssPluginWithConfig || postcss_1.default(postcssConfig);
                }
                textCSS = postcssConfig ? postcssPluginWithConfig.process(src) : postcssNested.process(src);
                break;
        }
        return moduleTemplate.replace('%s', JSON.stringify(parser.getCSSSelectors(textCSS)));
    },
};
module.exports = moduleTransform;
